<?php

namespace HCM\FPA\Controllers;

use HCM\FPA\Services\DependenteVincService;
use HCM\Geral\Services\ESocialService;
use HCM\Geral\Services\PessoaService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;

class DependenteVincController extends Crud {

    protected $dataSourceName = 'fpa_depvinculo';
    protected $dataSourceNameGet = 'fpa_depvinculo_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, DependenteVincService $dependenteVincService,  PessoaService $pessoaService, ESocialService $eSocialService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->dependenteVincService = $dependenteVincService;
        $this->pessoaService = $pessoaService;
        $this->eSocialService = $eSocialService;
        parent::__construct($dataSourceManager);
    }

    public function getDepvinculo (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
        $rows = $dataSet->getRows();
        
        foreach($rows as $key => &$row){
            if($row['NRTIPODEPENDE']){
                $row['NRTIPODEPENDE_LIST'] = explode(",", $row['NRTIPODEPENDE']);
                foreach($row['NRTIPODEPENDE_LIST'] as $chave => $value){
                    $row['NRTIPODEPENDE_LIST'][$chave] = intval($value);
                }
                $row['NRTIPODEPENDE'] = implode(",", $row['NRTIPODEPENDE_LIST']);
            }

            $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
        }
    }

    public function setDepvinculo (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $mensagem = '';
                $nrtipodeps = '';
                if (!is_array($row['NRTIPODEPENDE'])) {
                    $nrtipodeps = explode(",", $row['NRTIPODEPENDE']);
                }
                foreach($nrtipodeps as $nrtipodep){
                    $row['NRTIPODEPENDE'] = $nrtipodep;
                    $nrtipodepende = $row['NRTIPODEPENDE'];
                    $nrorg = $row['NRORG'];
                    $nrdepvinculo = $row['NRDEPVINCULO'];
                    $nrvinculom = $row['NRVINCULOM'];
                    $nrpessoa = $row['NRPESSOA'];
                    
                    $this->dependenteVincService->verificaExisteDependencia($nrorg, $nrdepvinculo, $nrvinculom, $nrpessoa, $nrtipodepende);
                    
                    if(isset($row['NRCPFPESSOA'])) {
                        $row['NRCPFPESSOA'] = str_replace(array('.','/','-'),'',$row['NRCPFPESSOA']);
                        $mensagem = $this->pessoaService->validaCpfExistePessoa($row, false);
                        $this->dependenteVincService->alteraCPFDpendente($nrpessoa, $row['NRCPFPESSOA'], $nrorg);
                    }
                    if (!isset($row['NREXCEDEPENDE'])) {
                        $row['NREXCEDEPENDE'] = '';
                    }
                    if (!isset($row['NROUTRVINCULOM'])) {
                        $row['NROUTRVINCULOM'] = '';
                    }
                    if (!isset($row['IDUNIVERSITARIO'])) {
                        $row['IDUNIVERSITARIO'] = '';
                    }

                    if (array_key_exists('NRDEPVINCULO', $row)) {
                        if ($row['NRDEPVINCULO'] === '' || sizeof($nrtipodeps)>1) {
                            $row['NRDEPVINCULO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_DEPVINCULO', 1, 12, true);
                        }
                    } else {
                        $row['NRDEPVINCULO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_DEPVINCULO', 1, 12, true);
                    }
                    
                    /*if (!is_array($nrtipodeps) && array_key_exists('NRDEPVINCULO', $row) && $row['NRDEPVINCULO'] === '') {
                        $row['NRDEPVINCULO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_DEPVINCULO', 1, 12, true);
                    } else {
                        $row['NRDEPVINCULO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_DEPVINCULO', 1, 12, true);
                    }*/

                    parent::save($request, $response);
                    
                    if(array_key_exists('IDENVIAESOCIAL', $row) && $row['IDENVIAESOCIAL'] == 'S'){
                        $this->eSocialService->envioEsocialDescentralizado($row, $response);
                    }
                    
                    if($mensagem){
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($mensagem, DTO\Response\Message::TYPE_WARNING));
                    }
                }

            }
        } catch (\Exception $e) {
            if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function deleteDepvinculo(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function cadPessoaDepend(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $mensagem = '';
                $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
                $nrvinculom = $row['NRVINCULOM'];
                $nrpessoavinc = $row['NRPESSOAVINC'];
                $nrtiporelaciona = $row['NRTIPORELACIONA'];
                $nmpessoa = $row['NMPESSOA'];
                $idsexopessoa = $row['IDSEXOPESSOA'];
                $dtnascpessoa = isset($row['DTNASCIFUNDPARC'])?$row['DTNASCIFUNDPARC']:null;
                $cdestacivil = isset($row['CDESTACIVIL'])?$row['CDESTACIVIL']:'';
                $nrnacionalid = isset($row['NRNACIONALID'])?$row['NRNACIONALID']:null;
                $cdpais = isset($row['CDPAIS'])?$row['CDPAIS']:'';
                $sgestado = isset($row['SGESTADO'])?$row['SGESTADO']:'';
                $cdmunicipio = isset($row['CDMUNICIPIO'])?$row['CDMUNICIPIO']:'';
                $dtfimrelacionamento = isset($row['DTFIMRELACIONAMENTO'])?$row['DTFIMRELACIONAMENTO']:null;
                $cpfpessoa = isset($row['NRCPFPESSOA'])? str_replace(array('.','/','-'),'',$row['NRCPFPESSOA']) : null;
                if($cpfpessoa){
                    $row['NRPESSOA'] = array_key_exists('NRPESSOA', $row) && !empty($row['NRPESSOA']) ? $row['NRPESSOA'] : null; 
                    $row['NRCPFPESSOA'] = str_replace(array('.','/','-'),'',$row['NRCPFPESSOA']);
                    $mensagem = $this->pessoaService->validaCpfExistePessoa($row,false);
                }
                
                if(!empty($mensagem)){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($mensagem, DTO\Response\Message::TYPE_WARNING));
                }else{
                    $pessoa = $this->dependenteVincService->realizaCadastroPessoaDependente($nrvinculom, $nrpessoavinc, $nmpessoa, $cdmunicipio, $sgestado, $cdpais, $nrnacionalid,
                        $cdestacivil, $idsexopessoa, $dtnascpessoa, $nrtiporelaciona, $dtfimrelacionamento,$cpfpessoa);
                }
                $response->addNotification(new DTO\Response\Notification('cadPessoaDepend', DTO\Response\Notification::TYPE_SUCCESS));
            }
        } catch(\Exception $e) {
             if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function cadAutoDependentes(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $nrvinculom = $row['NRVINCULOM'];
                $dependentes = array();
                foreach ($row['NRPESSOA'] as $index=>$nrpessoadep) {
                    $dependentes[] = array(
                        'NRORG' => $nrorg,
                        'NRPARCDEPENDE' => $row['NRPARCDEPENDE'][$index],
                        'NRTIPORELACIONA' => $row['NRTIPORELACIONA'][$index]
                    );
                }
                $this->dependenteVincService->cadastroAutomaticoDependentes($nrorg, $nrvinculom, $dependentes);

                $response->addNotification(new DTO\Response\Notification('cadAutoDependentes', DTO\Response\Notification::TYPE_SUCCESS));
            }
        } catch(\Exception $e) {
             if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}