<?php
namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;

class LancamentoEventoController extends Crud {
    
    protected $dataSourceName = 'fpa_itecalcfolha';
    protected $dataSourceNameGet = 'getItecalcfolha';
    protected $lancamentoEventoService;
    protected $itemLoteLancService;

    protected $parameterBagColumns = array();
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\FPA\Services\LancamentoEventoService $lancamentoEventoService, \HCM\FPA\Services\ItemLoteLancService $itemLoteLancService){
        $this->parameterBag    = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->lancamentoEventoService = $lancamentoEventoService;
        $this->itemLoteLancService = $itemLoteLancService;
        parent::__construct($dataSourceManager);
    }
    
    public function geraItemCalculoFolha(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $dtCompetencia = $this->hcmParameters->getDtCompetencia();
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $row = array();
            $dataset = array();
            foreach($conditions as $condition){
                $row[$condition['columnName']] = $condition['value'];
            }
            
            if(array_key_exists('LIST_NRESTRUTLEGAL', $row) || array_key_exists('LIST_NRESTRUTGEREN', $row) || array_key_exists('LIST_NRESTRUTSIND', $row) || array_key_exists('NRVINCULOM_LIST', $row)){
            
                $row['LIST_NRESTRUTLEGAL'] = empty($row['LIST_NRESTRUTLEGAL']) ? null : $row['LIST_NRESTRUTLEGAL']; 
                $row['LIST_NRESTRUTGEREN'] = empty($row['LIST_NRESTRUTGEREN']) ? null : $row['LIST_NRESTRUTGEREN']; 
                $row['LIST_NRESTRUTSIND'] = empty($row['LIST_NRESTRUTSIND']) ? null : $row['LIST_NRESTRUTSIND']; 
                $row['NRVINCULOM_LIST'] = empty($row['NRVINCULOM_LIST']) ? null : $row['NRVINCULOM_LIST'];
                $row['IDREFEVALORCALC'] = empty($row['IDREFEVALORCALC']) ? 'NORMAL' : $row['IDREFEVALORCALC'];
                $row['DTOCORRENCIA'] = empty($row['DTOCORRENCIA']) ? $dtCompetencia : $row['DTOCORRENCIA'];
                $row['QTOCORPROXPERI'] = empty($row['QTOCORPROXPERI']) ? 0 : $row['QTOCORPROXPERI'];
                $row['QTOCORPROXPERI'] = str_replace('%', '', $row['QTOCORPROXPERI']);
                
                // variáveis do filtro
                
                $vinc = array(); 
                $vrreffolha = 0;
                $vreventofolha = 0;
                
                if(array_key_exists('VRREFFOLHA', $row)){
                    $vrreffolha = str_replace('%', '', $row['VRREFFOLHA']);
                    $vrreffolha = str_replace('.', '', $vrreffolha);
                    $vrreffolha = str_replace(',', '.', $vrreffolha);
                    $vrreffolha = (float)($vrreffolha);
                }
                if(array_key_exists('VREVENTOFOLHA', $row)){
                    $vreventofolha = str_replace('%', '', $row['VREVENTOFOLHA']);
                    $vreventofolha = str_replace('.', '', $vreventofolha);
                    $vreventofolha = str_replace(',', '.', $vreventofolha);
                    $vreventofolha = (float)($vreventofolha);
                }else{
                    $vreventofolha = 0;
                }
                
                // Seleciona os viculos da estrutura.
                $dataset = array('NRVINCULOM_LIST'   => $row['NRVINCULOM_LIST'],
                                    'LIST_NRESTRUTLEGAL' => $row['LIST_NRESTRUTLEGAL'],
                                    'LIST_NRESTRUTGEREN' => $row['LIST_NRESTRUTGEREN'],
                                    'LIST_NRESTRUTSIND'  => $row['LIST_NRESTRUTSIND']);
                                    
                $vinculos = $this->itemLoteLancService->retornaVinculos($dataset);
                // Retorna os eventos selecionados por vínculo.
                if($vinculos){
                    $retorno = $this->lancamentoEventoService->getEventoPorVinculo($vinculos, $row['NREVENTO_LIST'], $nrorg, $dtCompetencia, $row['NRTIPOMOVIMENT'], $vrreffolha, $vreventofolha, $row['IDREFEVALORCALC'], $row['DTOCORRENCIA'], $row['QTOCORPROXPERI']);
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('geraItemCalculoFolha', $retorno));
                }else{
                   $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('geraItemCalculoFolha', []));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        
        }
        
    }
    
    public function deleteItemCalculoFolha(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $row = array();
            $nrModalidadeCalc = $this->hcmParameters->getNrModalidadeCalculo();
            $nrOcorrenciaCalc = $this->hcmParameters->getNrOcorrenciaCalculo();
            foreach($dataset[0] as $item){
                $item['NRMODALIDADECALC'] = $nrModalidadeCalc;
                $item['NROCORRENCIACALC'] = $nrOcorrenciaCalc;
                array_push($row, $item);
            }
            $this->lancamentoEventoService->deleteItemCalculoFolha($row);    
            
            $message = new \Zeedhi\Framework\DTO\Response\Message('Delete dos Itens finalizado com sucesso', DTO\Response\Message::TYPE_MESSAGE);
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            }
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    public function setItemCalculoFolha(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            error_reporting(E_ERROR | E_PARSE);
            $rows = $request->getDataSet()->getRows();
            foreach($rows as $rowVinculos){
                $this->lancamentoEventoService->setItemCalculoFolha($rowVinculos, $response);
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function adicionaItemCalcFolha(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $data = $request->getDataSet()->getRows();
            $row = $data[0];
            $dtCompetencia = $this->hcmParameters->getDtCompetencia();
            $evento = [];
            array_push($evento, $row['NREVENTOM']);
            
            $dataset = array('NRVINCULOM_LIST'   => $row['NRVINCULOM'],
                            'LIST_NRESTRUTLEGAL' => NULL,
                            'LIST_NRESTRUTGEREN' => NULL,
                            'LIST_NRESTRUTSIND'  => NULL);
            
            $vinculos = $this->itemLoteLancService->retornaVinculos($dataset);
            
            $row['VREVENTOFOLHA'] = (float)$row['VREVENTOFOLHA'];
            
            $retorno = $this->lancamentoEventoService->getEventoPorVinculo($vinculos, $evento, $row['NRORG'], $dtCompetencia, $row['NRTIPOMOVIMENT'], $row['VRREFFOLHA'], $row['VREVENTOFOLHA'], $row['IDREFEVALORCALC'], $row['DTOCORRENCIA'], $row['QTOCORPROXPERI']);
            
            $retorno[0]['DTINCLUSAO'] = substr($retorno[0]['DTINCLUSAO'], 0,10);
            $retorno[0]['DTULTATU'] = substr($retorno[0]['DTULTATU'],0, 10);
            $retorno[0]['VREVENTOFOLHA'] = $row['VREVENTOFOLHA'];
            $retorno[0]['VRREFFOLHA'] = $row['VRREFFOLHA'];
            $retorno[0]['IDREFEVALORCALC'] = $row['IDREFEVALORCALC'];
            $retorno[0]['DTOCORRENCIA'] = $row['DTOCORRENCIA'];
            $retorno[0]['QTOCORPROXPERI'] = $row['QTOCORPROXPERI'];
            
            
            $this->lancamentoEventoService->setItemCalculoFolha($retorno, $response);
            
            $message = new \Zeedhi\Framework\DTO\Response\Message('Item salvo com sucesso!', DTO\Response\Message::TYPE_MESSAGE);
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}
