<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\FPA\Services\TipoIncidenciaService;
use Zeedhi\Framework\DataSource\DataSet;

class TipoIncidenciaController extends Crud {

    protected $dataSourceName_get = 'fpa_tipoincide_get';
    protected $dataSourceName_set = 'fpa_tipoincide_set';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, TipoIncidenciaService $tipoIncidenciaService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->tipoIncidenciaService = $tipoIncidenciaService;
        parent::__construct($dataSourceManager);
    }

    public function getTipoIncidencia(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_get;
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        foreach($rows as $key => &$row){
            if(isset($row['DSNRTIPOMOVIMENT'])){
                $row['DSNRTIPOMOVIMENT'] = explode(",", $row['DSNRTIPOMOVIMENT']);
                
                foreach($row['DSNRTIPOMOVIMENT'] as $chave => $value)
                    $row['DSNRTIPOMOVIMENT'][$chave] = intval($value);
                    
                $row['NMTIPOMOVIMENT'] = explode(",", $row['NMTIPOMOVIMENT']);
            }    
        }
        $response->addDataSet(new DataSet($this->dataSourceName, $rows));
    }
    
    public function setTipoIncidencia(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_set;
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                if (array_key_exists('NRTIPOINCIDE', $row)) {
                    if ($row['NRTIPOINCIDE'] === '') {
                        $row['NRTIPOINCIDE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_TIPOINCIDE', 1, 12, true); 
                    }
                } else {
                    $row['NRTIPOINCIDE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_TIPOINCIDE', 1, 12, true); 
                }
                if(!array_key_exists('NMABREVTIPOINC', $row)){
                    $row['NMABREVTIPOINC'] = null;
                }
                
                $this->tipoIncidenciaService->validaCamposVazios($row);
                
                if(isset($row['DSNRTIPOMOVIMENT']) && is_array($row['DSNRTIPOMOVIMENT']) )
                    $row['DSNRTIPOMOVIMENT'] = implode(",", $row['DSNRTIPOMOVIMENT']);
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteTipoIncidencia(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceName_set;
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 