<?php

namespace HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\santander\registro;

use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\RegistroHolerite;
use HCM\Util\DateUtil;

class Registro0HoleriteSantander extends RegistroHolerite{

    const REGISTRO_0 = '0';

    const ID_TRANSMISSAO_ENVIO = 'E';
    const ID_TRANSMISSAO_RETORNO = 'R';

    private $_estrutura;
    private $_nrSeqArquivo;
    private $_nrSeqRegistro;
    private $_codConvenioBanco;
    private $_dtPagamento;
    private $_codBanco;
    private $_idTpArquivo;
    private $_nrLote;
    private $_idTpPagamento;

    public function geraRegistro0(){
        $this->_geraRegistro();
    }

    private function _geraRegistro(){
        // Bloco 01 | 0001-0001 | Tipo de registro = 0.
        $this->atribuiDado(self::REGISTRO_0, 1);

        // Bloco 02 | 0002-0002 | Identificação do Tipo de Transmissão = E.
        $this->atribuiDado(self::ID_TRANSMISSAO_ENVIO, 1);

        // Bloco 03 | 0003-00022 | Número do convênio.
        $this->atribuiDado($this->_codConvenioBanco, 20, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 04 | 0023-0023 | Tipo de arquivo.
        $this->atribuiDado($this->_idTpArquivo, 1, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 05 | 0024-0029 | Número de versão - Controle de sequência de envio do arquivo.
        $this->atribuiDadoEspecial($this->_nrSeqArquivo, 6, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 06 | 0030-0035 | Número de lote.
        $this->atribuiDadoEspecial($this->_nrLote, 6, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 07 | 0036-0082 | Nome da Empresa.
        $this->atribuiDado($this->_estrutura->historico->getNmrazsocestrut(), 47, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 08 | 0083-0096 | CNPJ.
        $this->atribuiDado($this->_estrutura->historico->getCdcnpjestrut(), 14, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 09 | 0097-0102 | Data de referência do pagamento (Formato AAAAMM).
        $this->atribuiDataReferencia();

        // Bloco 10 | 0103-0110 | Data de crédito (Formato AAAAMMDD).
        $aux = DateUtil::getDataDeString($this->_dtPagamento,DateUtil::FORMATO_BRASILEIRO,true)->format('Ymd');
        $this->atribuiDado($aux, 8, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);
    
        // Bloco 11 | 0111-0118 | Data de disponibilização (Formato AAAAMMDD).
        $aux = DateUtil::getDataDeString($this->_dtPagamento,DateUtil::FORMATO_BRASILEIRO,true)->format('Ymd');
        $this->atribuiDado($aux, 8, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 12 | 0119-0126 | Data de geração do arquivo.
        $this->atribuiDataCorrente();

        // Bloco 13 | 0127-0132 | Hora de geração do arquivo.
        $this->atribuiHoraCorrente();

        // Bloco 14 | 0133-0136 | Código do Banco utilizado na Empresa (0008, 0033, 0353).
        $this->atribuiDado($this->_codBanco, 4, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 15 | 0137-0140 | Código da Agência da Empresa.
        $this->atribuiDado($this->_estrutura->agencia, 4, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 16 | 0141-0152 | Conta Corrente da Empresa.
        $this->atribuiDado($this->_estrutura->contaCorrente, 12, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 17 | 0153-0153 | Identificação do tipo de pagamento.
        $this->atribuiDado($this->_idTpPagamento, 1, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 18 | 0154-0400 | Livre para futuras utilizações.
        $this->atribuiDado(self::CARACTER_BRANCO, 247, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

    }


    public function setEstrutura($estrutura){
        $this->_estrutura = $estrutura;
    }


    public function setSequencia($sequencia){
        $this->_sequencia = $sequencia;
    }


    public function setCodConvenioBanco($codConvenioBanco){
        $this->_codConvenioBanco = $codConvenioBanco;
    }


    public function setDtPagamento($dtPagamento){
        $this->_dtPagamento = $dtPagamento;
    }


    public function setCodBanco($codigo){
        $this->_codBanco = $codigo;
    }


    public function setTpArquivo($idtparquivo){
        $this->_idTpArquivo = $idtparquivo;
    }


    public function setNrLote($lote){
        $this->_nrLote = $lote;
    }


    public function setNrseqarquivo($nrsequencia) {
        $this->_nrSeqArquivo = $nrsequencia;
    }


    public function setIdtppagamento($idtppagamento) {
        $this->_idTpPagamento = $idtppagamento;
    }

    private function atribuiDataCorrente(){

        $dataAtual = DateUtil::getDataAtual();
        $dtAtual = $dataAtual->format('Ymd');

        $this->atribuiDado($dtAtual, 8);
    }


    private function atribuiHoraCorrente(){

        $horaAtual = DateUtil::getDataAtual(false)->format('Hms');
        $this->atribuiDado($horaAtual, 6);
    }

    private function atribuiDataReferencia(){
        $dataPgto = DateUtil::getDataDeString($this->_dtPagamento);
        $dataRef = $dataPgto->format('Ym'); 

        $this->atribuiDado($dataRef, 6);
    }

}