<?php
//define('HCM_ERROR_REPORTING', (E_ALL | E_STRICT) ^ (E_WARNING | E_NOTICE));
require_once __DIR__ . '/../../backend/bootstrap.php';

function main() 
{
    $argv = $_SERVER["argv"];
    $versao = '2.4.01';
    echo "Gerando $argv[1] para versao $versao\n";
    if ($argv[1] == null) {
        echo "[AVISO] Favor informar o nome do evento apos o comando ou ALL para todos\n";
        die;
    }
    else {
        $nmevento = $argv[1];

        $templateDir = __DIR__ . "/../../backend/files/esocial/datasource";

        // Busca o(s) evento(s) para fazer a geração
        $eventos = GpeEsocial::getEventos($nmevento, $versao);
        $template = array();
        $cont = 0;
        foreach ($eventos as $evento) {
            echo "[INFO] Geração do helper {$evento['NMEVENTO']}\n";
            $nomeEvento = $evento["NMEVENTO"];
            $versaoEvento = $evento["DSVERSAO"];
            $dadosEvento = GpeEsocial::getDadosEvento($evento["NREVENTO"]);

            $descricaoBuilder = new DescricaoDoArquivo (
                $nomeEvento, $versaoEvento, $dadosEvento
            );
            $descricaoDoEvento = $descricaoBuilder->buildData();

            $jsonBuilder = new Helpers($descricaoDoEvento);
            $template[$nomeEvento] = $jsonBuilder->build();

            $cont++;
        }

        if ($versaoEvento) {
            $diretorioVersao = str_replace(".", "-", $versaoEvento);
            if (!is_dir($templateDir)) {
                mkdir($templateDir);
                chmod($templateDir, "777");
                echo "[INFO] Criado diretorio " . $templateDir . "\n";
            }

            if (file_exists($templateDir . "/helper2_v_$diretorioVersao.json")) {
                echo "[AVISO] Ja existia o arquivo helper2_v_$diretorioVersao.json que foi substituido por um novo\n";
            }
            // Para gerar minificado retirar a propriedade JSON_PRETTY_PRINT
            file_put_contents($templateDir . "/helper2_v_$diretorioVersao.json", trim(json_encode($template, JSON_PRETTY_PRINT), '[]'));
        }

        echo "Criado(s) $cont template(s) json\n";
    }
}

class GpeEsocial
{
    public static function getEventos($evento, $versao)
    {
        $sql = "select *
                  from eso_evento
                 where idativo = 'S'
                   and ((nmevento = '$evento') or (upper('$evento') = 'ALL'))
                   and dsversao = '$versao'
                 order by nmevento";

        return \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql);
    }

    public static function getDadosEvento($nrevento)
    {
        $sql = "select eso_registro.nrregistro,
                       eso_registro.nrcampo, eso_registro.nmcampo,
                       eso_registro.nrcampopai, eso_registro.nmcampopai,
                       eso_registro.dselemento, eso_registro.dstipo , eso_registro.dsocorrencia, eso_registro.nrtamanho,
                       eso_registro.nrdecimais, eso_registro.dsregistro
              from eso_registro
             where eso_registro.nrevento = :NREVENTO
             order by eso_registro.nrregistro";
        $sql = "SELECT teste.*, rownum from ($sql) teste";

        return \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, array(":NREVENTO" => $nrevento));
    }
}

class DescricaoDoArquivo
{

    private $arquivo;
    private $versao;
    private $elementos;

    private $raiz;

    private $builtObj;

    const ATRIBUTO = "A";
    const ELEMENTO = "E";
    const GRUPO    = "G";

    /**
     * Metodo construtor.
     *
     * @param $arquivo   String Nome do arquivo especificado do arquivo de integracao (ex.: S-1100, para o e-Social).
     * @param $versao    String Versão do arquivo, em relação ao manual onde foi especificado (ex.: 1.1).
     * @param $elementos array  Array com os dados dos elementos que compõe o arquivo. Cada elemento deste array deverá
     *                          apresentar a seguinte estrutura:
     *                              "NRCAMPO"    : "<numero_do_elemento>",
     *                              "NMCAMPO"    : "<nome_do_elemento>",
     *                              "NRCAMPOPAI" : "<numero_do_elemento_pai>",
     *                              "NMCAMPOPAI" : "<nome_do_elemento_pai>",
     *                              "ELEMENTO"   : "<G,E,A (grupo, elemento ou atributo)>",
     *                              "TIPO"       : "<null, D, C, N (char, number, date)>",
     *                              "OCORRENCIA" : "<ocorrencia_do_elemento> (ex.: 1-1)",
     *                              "TAMANHO"    : "<tamanho_do_elemento>",
     *                              "DECIMAIS"   : "<numero_de_casas_decimais_do_elemento>",
     *                              "DESCRICAO"  : "<descricao_do_elemento>",
     * @param $raiz      String Especifica o nome do elemento raiz do arquivo. Caso nada seja passado, o builder deverá
     *                          pesquisar dentre os elementos aquele que nao possui o atributo "pai" e assumir este como
     *                          a raiz do arquivo.
     *
     * @return void
     */
    public function __construct($arquivo, $versao, $elementos = array(), $raiz = null)
    {
        $this->arquivo   = $arquivo;
        $this->versao    = $versao;
        $this->elementos = $elementos;
        $this->raiz      = $raiz;
    }


    public function buildData()
    {
        $dataCriacaoArquivo = new \DateTime();

        $this->builtObj["__raiz"]     = $this->raiz ? : $this->procurarElementoRaiz();
        //$this->builtObj["__versao"]   = $this->versao;
        //$this->builtObj["__arquivo"]  = $this->arquivo;
        //$this->builtObj["__dataHora"] = $dataCriacaoArquivo->format("d/m/Y H:i:s");
        $this->buildComponent();
        $this->setElementosFilhos();
        return $this->builtObj;
    }

    private function setElementosFilhos()
    {
        foreach ($this->elementos as $elemento) {
            if ($elemento["NRCAMPOPAI"]) {
                if ($elemento["DSELEMENTO"] === self::ATRIBUTO) {
                    if (!in_array($elemento["NRCAMPO"], $this->builtObj[$elemento["NRCAMPOPAI"]]["atributos"])) {
                        $this->builtObj[$elemento["NRCAMPOPAI"]]["atributos"][] = $elemento["NRCAMPO"];
                    }
                } else {
                    if (!in_array($elemento["NRCAMPO"], $this->builtObj[$elemento["NRCAMPOPAI"]]["filhos"])) {
                        $this->builtObj[$elemento["NRCAMPOPAI"]]["filhos"][] = $elemento["NRCAMPO"];
                    }
                }
            }
        }
    }

    private function buildComponent()
    {
        foreach ($this->elementos as $elemento) {
            $this->builtObj[$elemento["NRCAMPO"]] = array (
                "nrElemento"    => $elemento["NRCAMPO"],
                "nmElemento"    => $elemento["NMCAMPO"],
                "nrElementoPai" => $elemento["NRCAMPOPAI"],
                "nmElementoPai" => $elemento["NMCAMPOPAI"],
                "tipo"          => $elemento["DSTIPO"],
                "elemento"      => $elemento["DSELEMENTO"],
                "tamanho"       => $elemento["NRTAMANHO"],
                "ocorrencia"    => $this->getOcorrencias($elemento["DSOCORRENCIA"]),
                "decimais"      => $elemento["NRDECIMAIS"],
                "validacoes"    => $this->getValidacoes($elemento["DSREGISTRO"]),
                "atributos"     => array(),
                "filhos"        => array(),
            );
        }
    }

    private function getOcorrencias($str)
    {
        list($menor, $maior) = explode("-", $str);
        return array(
            "menor" => (string) $menor,
            "maior" => (string) $maior,
        );
    }

    private function getValidacoes($descricao)
    {
        preg_match_all("/REGRA_\w+/", $descricao, $result);
        return $result[0];
    }

    private function procurarElementoRaiz()
    {
        foreach ($this->elementos as $elemento) {
            if ((bool)$elemento["NRCAMPOPAI"] === false) {
                return $elemento["NRCAMPO"];
            }
        }

        return null;
    }

}

class Helpers
{

    private $descricaoDoObjeto;
    private $template;

    function __construct($descricaoDoObjeto) {
        $this->descricaoDoObjeto = $descricaoDoObjeto;
        $this->template = array();
        $this->elementos = array();
    }

    public function build()
    {
        $this->template = $this->criaElemento($this->descricaoDoObjeto, "arquivo", $this->descricaoDoObjeto["__raiz"], null);
        return $this->template;
    }

    private function criaElemento($arquivo, $auxElemento, $elemento, $nomeCanonico) {
        $template = array();
        $elementos = array();
        $inclusao = true;
        $alteracao = false;
        $exclusao = false;
        
        foreach ($this->descricaoDoObjeto as $index => $objeto) {
            if ($index == "__raiz" || $index == $elemento || isset($elementos[$index])) {
                continue;
            }
            
            if ($objeto['nmElemento'] == 'alteracao') {
                $alteracao = true;
            }
            if ($objeto['nmElemento'] == 'exclusao') {
                $exclusao = true;
            }
            
            if ($objeto['elemento'] == "A" or $objeto['elemento'] == "E") {
                // Inclui campos que não sejam obrigatórios (ocorrência menor igual a zero) para que não sejam gerados desnecessariamente no xml caso estejam vazios
                // Inclui campos em que o pai é obrigatório (ocorrência menor maior ou igual a 1)
                if ($objeto['ocorrencia']['menor'] == 0 || $arquivo[$objeto['nrElementoPai']]['ocorrencia']['menor'] >= 1) {
                    $nmElemento = strtoupper($objeto['nmElemento']);
                    if (array_key_exists($nmElemento, $template) && !$alteracao && !$exclusao) {
                        $nmElemento = strtoupper($arquivo[$objeto['nrElementoPai']]['nmElemento'] . '_' . $nmElemento);
                    }
                    $template[$nmElemento] = array(
                        'descricao' => $objeto['nmElemento'],
                        'ocorrencia' => array(
                            'menor' => $objeto['ocorrencia']['menor'],
                            'maior' => $objeto['ocorrencia']['maior']
                        )
                    );
                }
            }
            elseif ($objeto['ocorrencia']['maior'] > 1) {
                // Caso o elemento possa ocorrer mais de uma vez inclui ele com seus filhos em um subnível para que seja feita a validação no loop interno
                $nmElemento = strtoupper($objeto['nmElemento']);
                if (array_key_exists($nmElemento, $template) && !$alteracao && !$exclusao) {
                    $nmElemento = strtoupper($arquivo[$objeto['nrElementoPai']]['nmElemento'] . '_' . $nmElemento);
                }
                $template[$nmElemento] = array(
                    'descricao' => $objeto['nmElemento'],
                    'ocorrencia' => array(
                        'menor' => $objeto['ocorrencia']['menor'],
                        'maior' => $objeto['ocorrencia']['maior']
                    )
                );
                foreach ($objeto['filhos'] as $nrfilho) {
                    $nmFilho = strtoupper($arquivo[$nrfilho]['nmElemento']);
                    if (array_key_exists($nmFilho, $template[$nmElemento])) {
                        $nmFilho = strtoupper($arquivo[$arquivo[$nrfilho]['nrElementoPai']]['nmElemento'] . '_' . $nmFilho);
                    }
                    $template[$nmElemento][$nmFilho] = array(
                        'descricao' => $arquivo[$nrfilho]['nmElemento'],
                        'ocorrencia' => array(
                            'menor' => $arquivo[$nrfilho]['ocorrencia']['menor'],
                            'maior' => $arquivo[$nrfilho]['ocorrencia']['maior']
                        )
                    );
                    $elementos[$nrfilho] = $nmFilho;
                }
            }
        }
        
        return $template;
    }
}

main();